@@ -1,8 +1,6 @@
 package Git::PurePerl::Pack::WithIndex;
 use Moose;
 use MooseX::StrictConstructor;
-use namespace::autoclean;
-
 extends 'Git::PurePerl::Pack';
 
 has 'index_filename' =>
@@ -10,6 +8,8 @@ has 'index_filename' =>
 has 'index' =>
     ( is => 'rw', isa => 'Git::PurePerl::PackIndex', required => 0 );
 
+__PACKAGE__->meta->make_immutable;
+
 sub BUILD {
     my $self = shift;
     my $index_filename = $self->filename;
@@ -17,7 +17,6 @@ sub BUILD {
     $self->index_filename($index_filename);
 
     my $index_fh = IO::File->new($index_filename) || confess($!);
-    $index_fh->binmode();
     $index_fh->read( my $signature, 4 );
     $index_fh->read( my $version,   4 );
     $version = unpack( 'N', $version );
@@ -49,5 +48,4 @@ sub get_object {
     return $self->unpack_object($offset);
 }
 
-__PACKAGE__->meta->make_immutable;
-
+1;
