@@ -2,7 +2,6 @@ package Git::PurePerl::PackIndex;
 use Moose;
 use MooseX::StrictConstructor;
 use IO::File;
-use namespace::autoclean;
 
 has 'filename' =>
     ( is => 'ro', isa => 'Path::Class::File', required => 1, coerce => 1 );
@@ -13,6 +12,8 @@ has 'offsets' =>
     ( is => 'rw', isa => 'ArrayRef[Int]', required => 0, auto_deref => 1, );
 has 'size' => ( is => 'rw', isa => 'Int', required => 0 );
 
+__PACKAGE__->meta->make_immutable;
+
 my $FanOutCount   = 256;
 my $SHA1Size      = 20;
 my $IdxOffsetSize = 4;
@@ -42,5 +43,4 @@ sub BUILD {
     $self->size( $offsets[-1] );
 }
 
-__PACKAGE__->meta->make_immutable;
-
+1;
